
// ******************************************************************
//
// Plotting a histogram with AIDA and Grace
// ========================================
//
// This simple program plots a 1D histogram stored in a XML file that
// follows the AIDA Document Type Definition DTD:  
// http://aida.freehep.org/schemas/3.2.1/aida.dtd
//
// The program takes two arguments:
//           1. The XML file name 
//           2. The name of the IHistogram1D object to be plot
//
// The output is a postscript file (created with Grace) called
// histogram.ps.
//
// The program consists of a main function, which calls the actual
// plotting function.
//
// ******************************************************************

#include "AIDA/AIDA.h"
#include <iostream>
#include <string>
#include <vector>


int plot(std::string fileName, std::string histoName) {

  // ******************************************************
  // PART I: Creating factories and the object tree
  // ******************************************************
  
  // An AIDA analysis factory is created:
  AIDA::IAnalysisFactory* srcAnalysisFactory = AIDA_createAnalysisFactory();

  // The analysis factory is used to create a tree factory:
  AIDA::ITreeFactory* srcTreeFactory = 
                                  srcAnalysisFactory -> createTreeFactory();

  // The tree factory is used to create a tree, to which a store is
  // assigned (file stream): 
  bool readOnly = true;
  bool createNew = false;
  AIDA::ITree* srcTree = 
              srcTreeFactory -> create(fileName.c_str(),// file name
                                       "XML",           // format is XML
                                       readOnly,      // file opened read-only 
                                       createNew);    // do not create new file

  if(!srcTree) { 
     std::cerr << "Error: Cannot create XML tree" << std::endl;
  }

  // The tree factory can now be deleted
  delete srcTreeFactory; 

  // ******************************************************
  // PART II: Searching the histogram in file
  // ******************************************************

  // The object names and object types are retrieved from the tree into
  // vectors:
  std::vector<std::string> objectList = srcTree -> listObjectNames("/"),
                           typeList = srcTree -> listObjectTypes("/");
  // Note: For a given object name (in the vector objectList), the object
  //       type can be retrieved from typeList by using the same vector index
  //       (See the loop below).

  // The base type of an AIDA::IHistogram1D is AIDA::IManagedObject:
  AIDA::IManagedObject* srcObject = 0;

  // We loop through all the object names and types and check if the
  // histogram specified by the user exists:
  size_t nmbObjects = objectList.size();  
  for( int i = 0 ; i < nmbObjects; i++) {

      // If the histogram name was found and the type is correct (i.e.
      // IHistogram1D), the pointer to the histogram is retrieved:
      if(objectList[i] == histoName && typeList[i] == "IHistogram1D") {
         srcObject = srcTree -> find(histoName.c_str()); 
      }
  }
  
  // We check if the histogram retrieval was successful:
  if(!srcObject) {
     std::cerr << "Failed retrieving histogram " << histoName << " from file."
               << std::endl;
     // If not we exit:
     return 1;
  }
  else {
     std::cout << "Histogram " << histoName << " successfully retrieved."
               << std::endl;
  }
 
  // Note: The object is of type IManagedOject. We need apply the cast 
  // operator to get an IHistogram1D object:
  AIDA::IHistogram1D* histogram = dynamic_cast<AIDA::IHistogram1D*>(srcObject);

  
  // *********************************************
  // PART III: Plotting the histogram
  // *********************************************
  
  // The analysis factory is used to create a plotter factory:
  AIDA::IPlotterFactory* plotterFactory 
                       = srcAnalysisFactory -> createPlotterFactory();

  // Then, the plotter factory is used to create a plotter:
  AIDA::IPlotter* plotter = plotterFactory -> create("Plot of Histogram"); 
 
  // The histogram is plotted and written to a file (in postscript format)
  plotter -> region(0) -> plot(*histogram);
  plotter -> writeToFile("histogram.ps","PS");

  // The plotter and the plotter factory is deleted:
  delete plotter;
  delete plotterFactory;

  // The tree is closed:
  srcTree -> commit();
  srcTree -> close();

  // Finally, the tree and the analysis factory are deleted:
  delete srcTree;
  delete srcAnalysisFactory;

  return 0;
}



int main(int argc,char** argv) {

  // ******************************************************
  // The program expects two arguments: 
  //   1) The XML file name (created with AIDA)
  //   2) The histogram name (of the IHistogram1D object)
  // ******************************************************

  // An error message is printed if the number of arguments is wrong:
  if(argc != 3) {
     std::cerr << "Error. Wrong number of arguments." 
               << std::endl;
     std::cout << "Usage " << argv[0] 
               << " <file name> <histogram name>" 
               << std::endl; 
     return 1;
  }

  // The file and histogram names are assigned to variables:
  std::string fileName = argv[1];
  std::string histoName = argv[2];

  return plot(fileName,histoName);
}


